<?php

/**
 * @file
 * Contains \Drupal\publish_away\Plugin\Field\FieldWidget\PublishAwayWidget.
 */

namespace Drupal\publish_away\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;
use Drupal\Core\Url;
use Drupal\image\Plugin\Field\FieldWidget\ImageWidget as ImageWidget;

/**
 * Plugin implementation of the 'publish_away' widget.
 *
 * @FieldWidget(
 *   id = "publish_away",
 *   label = @Translation("Publish away"),
 *   field_types = {
 *     "publish_away"
 *   }
 * )
 */
class PublishAwayWidget extends FileWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'rows' => 3,
      'provider' => '',
      'max_length' => 115,
      'placeholder' => t('Message to be post on social network.'),
    ) + ImageWidget::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {die;
    $element = parent::settingsForm($form, $form_state);
    $element += ImageWidget::settingsForm($form, $form_state);

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $summary += ImageWidget::settingsSummary();
    
    // Message.
    $summary[] = t('Number of rows: !rows', array('!rows' => $this->getSetting('rows')));
    $placeholder = $this->getSetting('placeholder');
    if (!empty($placeholder)) {
      $summary[] = t('Placeholder: @placeholder', array('@placeholder' => $placeholder));
    }
    
    /*
    $summary = array();
    $settings = $this->getSettings();
    if (!empty($settings)) {
      foreach ($settings as $key => $value) {
        if (trim($value) === '') {
          continue;
        }
        $summary[] = t(ucfirst($key). ': @' . $key, array('@' . $key => $value));
      }
    }
    else {
      $summary[] = t('No placeholder');
    }*/

    return $summary;
  }

  /**
   * Overrides \Drupal\file\Plugin\Field\FieldWidget\FileWidget::formMultipleElements().
   *
   * Special handling for draggable multiple widgets and 'add more' button.
   */
  protected function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {
    $elements = parent::formMultipleElements($items, $form, $form_state);
    $elements += ImageWidget::formMultipleElements($items, $form, $form_state);

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['#access'] = \Drupal::currentUser()->hasPermission('use publish away');
    // Providers.
    $current_path = Url::fromRoute('<current>');
    $element['provider'] = array(
      '#type' => 'select',
      '#options' => array('empty' => t('- None -'), 'facebook' => 'Facebook', 'twitter' => 'Twitter'),
      // '#default_value' => $this->getSelectedOptions($items, $delta),
      // Do not display a 'multiple' select box if there is only one option.
      // '#multiple' => TRUE, // $this->multiple && count($this->options) > 1,
      '#access' => !\Drupal::service('path.matcher')->matchPath($current_path, '/admin/structure/types/manage/*/fields/*'),
      // '#title' => t('Provider(s)'),
      // '#default_value' => (int) FALSE,
      '#default_value' => isset($items[$delta]->provider) ? array($items[$delta]->provider) : NULL,
      '#description' => t("This option is explicitly disabled by default regardless of content's state."),
      '#weight' => -51,
      // '#empty_option' => $this->t('Source text only, no translations'),
      // '#empty_value' => LanguageInterface::LANGCODE_SYSTEM,
      // '#title_display' => 'invisible',
    );
    // Message.
    $element['message'] = array(
      '#type' => 'textarea',
      '#default_value' => $items[$delta]->message,
      '#rows' => $this->getSetting('rows'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#attributes' => array('class' => array('text-full')),
      '#weight' => -50,
      '#default_value' => '[node:title]',
    );
    // Display token browser.
    if (\Drupal::moduleHandler()->moduleExists('token')) {
      $token_types = array('node');
      $element['token_tree'] = array(
        '#type' => 'container',
        '#dialog' => TRUE,
        '#theme' => 'token_tree', 
        '#token_types' => $token_types,
      );
    }
    else {
      $element['token_tree'] = array(
        '#markup' => '<p>' . t('Enable the <a href="@drupal-token">Token module</a> to view the available token browser.', array('@drupal-token' => 'http://drupal.org/project/token')) . '</p>',
      );
    }

    $element += parent::formElement($items, $delta, $element, $form, $form_state);
    $element += ImageWidget::formElement($items, $delta, $element, $form, $form_state);

    if (empty($element['#default_value']['fids']) && !empty($element['#default_image']['fid'])) {
      // $element['#default_value']['fids'] = array($element['#default_image']['fid']);
    }

    return $element;
  }

  /**
   * Form API callback: Processes a image_image field element.
   *
   * Expands the image_image type to include the alt and title fields.
   *
   * This method is assigned as a #process callback in formElement() method.
   */
  public static function process($element, FormStateInterface $form_state, $form) {
    return ImageWidget::process($element, $form_state, $form);
  }

  /**
   * Validate callback for alt and title field, if the user wants them required.
   *
   * This is separated in a validate function instead of a #required flag to
   * avoid being validated on the process callback.
   */
  public static function validateRequiredFields($element, FormStateInterface $form_state) {
    ImageWidget::validateRequiredFields($element, $form_state);
  }

}
